from setuptools import setup

setup(name='lbppy',
      version='0.0.0',
      description='python api for consuming SCTA data',
      url='http://github.com/lombardpress/lbppy',
      author='Jeffrey Witt',
      author_email='jeffreycwitt@gmail.com',
      license='MIT',
      packages=['lbppy'],
      zip_safe=False)
